from log_constants import LogConstants

class URLBuilder(object):
    def __init__(self, host, regex_list):
        self.__protocol = 'http'
        self.__host = host
        self.__regex_list = regex_list

    def build(self, request_data):
        request_line = request_data[LogConstants.REQUESTLINE].split(' ')
        path = request_line[1]
        url = self.__protocol + '://' + self.__host + path
        for regex_tuple in self.__regex_list:
            url = regex_tuple[0].sub(regex_tuple[1], url)
        return url


class ConfigConstants(object):
    HOST = 'host'
    STATUS = 'status'
    METHODS = 'methods'
    HEADER = 'header'
    TRANSFORM = 'transform'
    LOGFORMAT = 'log_format'
    THREADCOUNT = 'thread_count'
    LOGFILE = 'log_file'
    VERBOSE = 'verbose'
    CONFIGFILE = 'config_file'


import Queue
import urllib2
import logging
import threading

from inspector import Inspector

class HTTPWorker(threading.Thread):
    def __init__(self, thread_id, headers, request_queue, url_filter, url_builder):
        threading.Thread.__init__(self)
        self.__thread_id = thread_id
        self.__opener = urllib2.build_opener()
        self.__opener.addheaders = headers
        self.__request_queue = request_queue
        self.__url_filter = url_filter
        self.__url_builder = url_builder
        self.__do_work = True
        self.__inspector = Inspector()

    def run(self):
        logging.info('Start thread ' + str(self.__thread_id))
        while self.__do_work or (not self.__request_queue.empty()):
            try:
                data = self.__request_queue.get(True, 1)
                if self.__url_filter.proceed(data):
                    url = self.__url_builder.build(data)
                    response = self.__opener.open(url)
                    self.__inspector.compare(url, data, response)
                    response.close()
            except Queue.Empty:
                logging.info('Queue is empty for thread ' + str(self.__thread_id))
        logging.info('Stop thread ' + str(self.__thread_id))

    def get_inspector(self):
        return self.__inspector

    def stop(self):
        self.__do_work = False


from log_constants import LogConstants

class URLFilter(object):
    def __init__(self, codes, methods):
        self.__codes = codes
        self.__methods = methods

    def __check_methods(self, request_data):
        if self.__methods:
            request_line = request_data[LogConstants.REQUESTLINE]
            for method in self.__methods:
                if request_line.startswith(method + ' '):
                    return True
            return False
        else:
            return True

    def __check_codes(self, request_data):
        if self.__codes:
            status_code = request_data[LogConstants.STATUSCODE]
            if status_code in self.__codes:
                return True
            return False
        else:
            return True

    def proceed(self, request_data):
        return self.__check_methods(request_data) and self.__check_codes(request_data)


import logging
import os

from log_constants import LogConstants


class Inspector(object):
    def __init__(self):
        self.__count = 0
        self.__status_match = 0
        self.__size_match = 0

    def __add__(self, other):
        self.__count += other.__count
        self.__status_match += other.__status_match
        self.__size_match += other.__size_match
        return self

    def __str__(self):
        result = '[Overview]' + os.linesep
        result += 'Requests: ' + str(self.__count) + os.linesep
        result += '[Responses matched / mismatched]' + os.linesep
        result += "Status codes: " + str(self.__status_match) + ' / ' + str(
            self.__count - self.__status_match) + os.linesep
        result += "Size: " + str(self.__size_match) + ' / ' + str(self.__count - self.__size_match)

        return result

    def compare(self, url, log_data, response):
        self.__count += 1
        status = response.getcode()
        length = len(response.read())

        if str(status) == log_data[LogConstants.STATUSCODE]:
            self.__status_match += 1

        logging.debug('URL: ' + url + ' Length: ' + str(length) + ' Status: ' + str(status))


from exceptions import IOError
import os
import re

import ConfigParser

from config_constants import ConfigConstants

class ConfigReader(object):
    def __init__(self, config_file):
        self.__config = {}
        if config_file is not None:
            if os.path.isfile(config_file):
                self.__read_config_file(config_file)
            else:
                raise IOError('File ' + config_file + ' not found')

    def __read_config_file(self, config_file):
        cfgparser = ConfigParser.SafeConfigParser()
        cfgparser.optionxform = str
        cfgparser.read(config_file)
        self.__copy_value(cfgparser, 'General', 'LogFormat', ConfigConstants.LOGFORMAT, True)
        self.__copy_value(cfgparser, 'General', 'Host', ConfigConstants.HOST)
        self.__copy_list(cfgparser, 'Filter', 'Status', ConfigConstants.STATUS)
        self.__copy_list(cfgparser, 'Filter', 'Methods', ConfigConstants.METHODS)
        self.__copy_tuple(cfgparser, 'Header', ConfigConstants.HEADER)
        for section in cfgparser.sections():
            if section.startswith('Transform'):
                self.__copy_regex_tuple(cfgparser, section, 'search', 'replace', ConfigConstants.TRANSFORM)

    def __copy_tuple(self, config, section, target):
        if config.has_section(section):
            for option in config.options(section):
                free_tuple = (option, config.get(section, option))
                if target in self.__config:
                    self.__config[target].append(free_tuple)
                else:
                    self.__config[target] = [free_tuple]

    def __copy_regex_tuple(self, config, section, arg1, arg2, target):
        if config.has_section(section):
            regex = re.compile(config.get(section, arg1))
            replace = config.get(section, arg2)
            named_tuple = (regex, replace)
            if target in self.__config:
                self.__config[target].append(named_tuple)
            else:
                self.__config[target] = [named_tuple]

    def __copy_list(self, config, section, option, target):
        if config.has_option(section, option):
            value = config.get(section, option)
            self.__config[target] = value.split(",")

    def __copy_value(self, config, section, option, target, raw=False):
        if config.has_option(section, option):
            self.__config[target] = config.get(section, option, raw)

    def merge_dict(self, args):
        for key in args.keys():
            value = args[key]
            if value is not None:
                self.__config[key] = args[key]

    def get_config(self):
        return self.__config